<?php
error_reporting(E_ALL | E_STRICT);
require('ballot.php');
require('candidate.php');

function wright($incoming_candidates, $ballots) {
	global $NBR_SEATS;

	global $continuing_candidates;

	$continuing_candidates = $incoming_candidates;
	$substitute_candidates = array();
	


	$nbr_iterations = 0;

	//Specialhantera fallet att alla platser är fyllda från början
	if(count($continuing_candidates) == $NBR_SEATS){
		return array($incoming_candidates, array());
	}
	

	do{
		$nbr_iterations++;
		echo "Påbörjar iteration $nbr_iterations\n";

		//Reset variables
		foreach($continuing_candidates as $val){
			$val->reset();
		}

		$provisional_candidates = array();
		$total_vote = 0;

		/*************************************************************
		 * Distribution of Preferences and calculation of Quota
		 **************************************************************/

		//Updates ballot for next iteration and check for exhaust
		foreach($ballots as $key=>$val){
			if($val->nextIteration()){
				//echo "\tRöstsedel $key struken\n";
				unset($ballots[$key]);
			}
		}

		echo "Nu är det " . count($ballots) . " röster kvar.\n";

		//Calculates all Candidate's Total Value of Votes (Ctvv)
		foreach($ballots as $ballot){
			$preferences = $ballot->getPreferences();

			$first_preference = array_keys($preferences);
			$first_preference = array_shift($first_preference);
			$first_preference_value = array_shift($preferences);

			//The value should always be 1 since we set it in the loop before
			if($first_preference_value != 1)
				//throw new Exception('First preference value should be 1 when starting a new iteration');

				//The first preference is a candidate that is not left in the process, something has gone wrong somewhere
				if(!isset($continuing_candidates[$first_preference]))
					throw new Exception('First candidate is not continuing');

			//The value of the ballot should be 1 in the first iteration
			if($ballot->getValue() != 1)
				throw new Exception('The value of the ballot should be 1 whens tarting a new iteration.');

			$continuing_candidates[$first_preference]->addVote($ballot, $first_preference);
		}

		//Calculates the new Total Vote (Tv) value.
		$total_vote=0;
		foreach($continuing_candidates as $val){
			$total_vote += $val->getCtvv();
		}

		//Calculate new Quota
		//$quota = floor($total_vote/(1+$NBR_SEATS))+1;
		$quota = $total_vote/(1+$NBR_SEATS)+1;




		do {
			echo "\nDistribuerar ut överskott (surplus).\n";
			
			//Loop-villkoret
			$old_provisional_candidates = count($provisional_candidates);

			/*************************************************************
			 * Provisional declaration of elected candidates
			 **************************************************************/
			foreach($continuing_candidates as $key=>$val){
				if($val->getCtvv() >= $quota) {
					$provisional_candidates[$key] = $val;
					echo "\tKandidat nummer $key:{$val} har blivit preliminärt vald, med quota $quota och Ctvv {$val->getCtvv()}.\n";
				}
			}



			/*************************************************************
			 * Number of vacancies filled
			 **************************************************************/
			if(checkEnoughFilled($provisional_candidates)) break;


			/*************************************************************
			 * Number of elected candidates less than vacancies
			 **************************************************************/
			//Check if finished
			if(count($continuing_candidates) == $NBR_SEATS)
				break;

			/*************************************************************
			 * Calculation of the Candidate's Surplus Value
			 **************************************************************/
			uasort($provisional_candidates, 'compareCandidateCtvv');
			foreach($provisional_candidates as $candidate_id=>$candidate) {
				//Surplus value (Sv)
				$Sv = $candidate->getCtvv()-$quota;


				/*************************************************************
				 * Calculation of the Surplus Transfer Value and value of the vote
				 **************************************************************/
				foreach($candidate->getVotesOnMe() as $ballot) {
					$Stv = $Sv/$candidate->getCtvv()*$ballot->getCandidateValue($candidate_id); //Var tidigare getValue()
					$ballot->subtractBeforeSurplusTransferValue($candidate_id, $Stv);
				}

				/*************************************************************
				 * Distribution of Candidate's Surplus votes
				 **************************************************************/
				foreach($candidate->getVotesOnMe() as $ballot) {
					$ballot->doSurplusValueTransfer($provisional_candidates, $candidate_id);

				}


				/*************************************************************
				 * Allocate value of quota to candidate whose surplus has been distributed
				 **************************************************************/
				$candidate->setCtvv($quota);
			}

		} while($old_provisional_candidates != count($provisional_candidates));


		/*************************************************************
		 * Exclusion of candidates
		 **************************************************************/
		if(checkEnoughFilled($provisional_candidates)) break;

		uasort($continuing_candidates, 'compareCandidateCtvv');

		$excluded_candidate = array_pop($continuing_candidates);
		$substitute_candidates[] = $excluded_candidate;

		echo "\nPlockar bort $excluded_candidate med Ctvv {$excluded_candidate->getCtvv()}.\n";

		if(count($continuing_candidates) == $NBR_SEATS){
			$provisional_candidates = $continuing_candidates;
			break;
		}



		echo "\n----------------------------------------------------\n\n";
	}while(count($continuing_candidates) > $NBR_SEATS);



	/*************************************************************
	 * Conclussion of the count
	 **************************************************************/
	return array($provisional_candidates, $substitute_candidates);


}

/*
   Returns true if we have filled all the vacancies
 */
function checkEnoughFilled($provisional_candidates) {
	global $NBR_SEATS;

	if(count($provisional_candidates) == $NBR_SEATS){
		return true;
	} elseif(count($provisional_candidates) > $NBR_SEATS) {
		throw new Exception('More candidates were elected than there are number of seats.');
	}

	return false;
}

?>
